@@ -1,7 +1,7 @@
 ;;; yow.el --- quote random zippyisms
 
-;; Copyright (C) 1993, 1994, 1995, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+;; Copyright (C) 1993, 1994, 1995, 2000, 2001, 2002, 2003, 2004, 2005,
+;;   2006, 2007, 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
 
 ;; Maintainer: FSF
 ;; Author: Richard Mlynarik
@@ -91,7 +91,7 @@ If called interactively, display a list of matches."
         (setq i (1+ i))))
     (and matches
          (setq matches (sort matches 'string-lessp)))
-    (and (interactive-p)
+    (and (called-interactively-p 'interactive)
          (cond ((null matches)
                 (message "No matches found."))
                (t
@@ -101,7 +101,7 @@ If called interactively, display a list of matches."
                       (princ (car l))
                       (setq l (cdr l))
                       (and l (princ "\n\n")))
-		    (print-help-return-message))))))
+		    (help-print-return-message))))))
     matches))
 
 
