@@ -1,7 +1,7 @@
+/* Copyright    Massachusetts Institute of Technology    1985	*/
+
 #include "copyright.h"
 
-/* $Header: /cvsroot/emacs/emacs/oldXMenu/SetFrz.c,v 1.1 1999/10/03 19:35:13 fx Exp $ */
-/* Copyright    Massachusetts Institute of Technology    1985	*/
 
 /*
  * XMenu:	MIT Project Athena, X Window system menu package
@@ -26,3 +26,6 @@ XMenuSetFreeze(menu, freeze)
      */
     menu->freeze = freeze;
 }
+
+/* arch-tag: 69c5670b-3a46-4c78-8fdb-305936d79772
+   (do not change this comment) */
