@@ -1,6 +1,6 @@
 /* systime.h - System-dependent definitions for time manipulations.
    Copyright (C) 1993, 1994, 2002, 2003, 2004,
-                 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+                 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
